use std::str::FromStr;

use piet::kurbo::{Rect, Vec2};

pub fn parse_positive_int_vec2_from_str(s: &str, separator: char) -> Result<Vec2, ()> {
    let values = s.split_once(separator);

    if let Some((x, y)) = values {
        if let Some(x) = usize::from_str(x).ok() {
            if let Some(y) = usize::from_str(y).ok() {
                return Ok(Vec2::new(x as f64, y as f64));
            }
        }
    }

    Err(())
}

pub fn parse_positive_rect_from_str(s: &str) -> Result<Rect, ()> {
    let (pairs, is_corner_points_mode) = if let Some(pairs) = s.split_once('+') {
        (pairs, false)
    } else if let Some(pairs) = s.split_once('-') {
        (pairs, true)
    } else {
        return Err(());
    };

    let first_vec = parse_positive_int_vec2_from_str(pairs.0, 'x')?;
    let second_vec = parse_positive_int_vec2_from_str(pairs.1, 'x')?;

    Ok(if is_corner_points_mode {
        Rect::from_points(first_vec.to_point(), second_vec.to_point())
    } else {
        Rect::from_origin_size(first_vec.to_point(), second_vec.to_size())
    })
}
