use rgb::{RGB8, RGBA8};

pub fn parse_rgb8_from_hex_str(s: &str) -> Result<RGB8, ()> {
    let first_index = if s.starts_with('#') { 1 } else { 0 };
    if s.len() - first_index == 6 {
        let r = u8::from_str_radix(&s[first_index..(first_index + 2)], 16).map_err(|_| ())?;
        let g = u8::from_str_radix(&s[(first_index + 2)..(first_index + 4)], 16).map_err(|_| ())?;
        let b = u8::from_str_radix(&s[(first_index + 4)..(first_index + 6)], 16).map_err(|_| ())?;

        return Ok(RGB8::new(r, g, b));
    }

    Err(())
}

pub fn parse_rgba8_from_hex_str(s: &str) -> Result<RGBA8, ()> {
    let first_index = if s.starts_with('#') { 1 } else { 0 };
    if s.len() - first_index == 8 {
        let r = u8::from_str_radix(&s[first_index..(first_index + 2)], 16).map_err(|_| ())?;
        let g = u8::from_str_radix(&s[(first_index + 2)..(first_index + 4)], 16).map_err(|_| ())?;
        let b = u8::from_str_radix(&s[(first_index + 4)..(first_index + 6)], 16).map_err(|_| ())?;
        let a = u8::from_str_radix(&s[(first_index + 6)..(first_index + 8)], 16).map_err(|_| ())?;

        return Ok(RGBA8::new(r, g, b, a));
    }

    Err(())
}

pub fn parse_rgb8_with_optional_alpha_from_hex_str(s: &str, fallback_alpha: u8) -> Result<RGBA8, ()> {
    // optionally +1 for the '#'
    match s.len() {
        6 | 7 => Ok(parse_rgb8_from_hex_str(s)?.alpha(fallback_alpha)),
        8 | 9 => Ok(parse_rgba8_from_hex_str(s)?),
        _ => Err(()),
    }
}
